; -----------------------------------------------------------------------------
; Hunchy II - v0.2 - Atari 2600 VCS - 4K ROM
; Copyright (C) 2005 Chris Walton (cd-w) <cwalton@gmail.com>
; -----------------------------------------------------------------------------

; TODO:	
; 1) Implement Lives, Scoring & Bonus Timer
; 2) Fall Off Ladder When Jumping?
; 3) Vary Knight and Player Movement Speeds
; 4) Implement Rope Swing
; 5) Implement Title Screen
; 6) Fix Sound Effects
; 7) Use Memory More Wisely (Combine MMAX,MCOUNT etc.)
; 8) Move Movement Patters For Knights (e.g. Jumping & Roaming)
; 9) Dissolving Blocks
; 10) Improve Sprites Appearance
; 11) PAL Conversion

        PROCESSOR 6502
        INCLUDE vcs.h
        INCLUDE macro.h

        ; BIT Absolute Macro
        MAC BITABS
        .byte $2C
        ENDM

; Game Constants
NO_ILLEGAL_OPCODES = 1          ; No Cheating :-)
PALCOLS         = 0             ; PAL/NTSC Colours (0 = NTSC)
PALTIME         = 0             ; 50Hz/60Hz Timing (0 = 60Hz)

; Screen Constants
KERNH		= 96		; Screen Height
MAXROW		= 12		; Platform Rows
	
; Player Constants
PHEIGHT		= 11		; Sprite Height
XMIN		= 17		; Minimum X Position
XMAX		= 136		; Maximum X Position
STARTX		= 30		; Starting Position
STARTY		= 40
JUMPHEIGHT	= 10		; Jump height (Pixels)
		
; Knight Constants
KHEIGHT		= 12		; Sprite Height

; Wall Constants
WHEIGHT		= 8		; Data Length

; Knight Delay
KDELAY		= 2		; Delay Factor

; Missile Constants
MISSILEMAX      = 67            ; Maximum X Position
MISSILEMAX2 = MISSILEMAX + 128  ; Set Bit 7 
MISSILEMIN      = 4             ; Minimum X Position
MISSILEMIN2 = MISSILEMIN + 128  ; Set Bit 7	
		
        IF PALCOLS
; PAL Colours
BLACK           = $00
WHITE           = $0E
RED             = $62
GREEN           = $56
BLUE            = $D6
YELL            = $26
GREY            = $04
PURPLE          = $84
ORANGE          = $44
PINK            = $4C	
        ELSE    
; NTSC Colours
BLACK           = $00
WHITE           = $0C
RED             = $42
GREEN           = $C6
BLUE            = $84
YELL            = $16   
GREY            = $04
PURPLE          = $50
ORANGE          = $24
PINK            = $3C			
        ENDIF

        SEG.U VARS
        ORG $80

; General Game Variables
LEVEL		ds 1		; Game Level
LPTR		ds 2		; Pointer To Level Data
CYCLE		ds 1		; Game Cycle
RANDOM		ds 1		; Random Number Seed/Value
LTIME		ds 1		; Remaining Lives (Bits 5-7) & Time (Bits 0-4)
SCORE		ds 2		; High Score (BCD)
DUR0		ds 1		; Sound Duration (Channel 0)
SWCH		ds 1		; Console Switch Status
			
; Screen Variables
SCREEN		ds #MAXROW*4	; Screen Data
BBASE		ds 1		; Blocks MSB
ROW		ds 1		; Current Row (0->11)
SPTR0		ds 2		; PF1 Pointer (LHS)
SPTR1		ds 2		; PF2 Pointer (LHS)
SPTR2		ds 2		; PF2 Pointer (RHS)
SPTR3		ds 2		; PF1 Pointer (RHS)
END		ds 1		; Temp Screen Variable
			
; Player Variables
PLAYERX		ds 1		; Player X Coordinate 
PLAYERY		ds 1		; Player Y Coordinate
JSTEP		ds 1		; Jump Step
COLLIDE		ds 1		; Collision Vector
PSWITCH		ds 1		; Player SwitchDraw Start
PEND		ds 1		; Player SwitchDraw End
PPTR		ds 2		; Sprite Data Pointer
	
; Knight Variables
KNIGHTX		ds 1		; Knight X Coordinate
KNIGHTY		ds 1		; Knight Y Coordinate	
KSWITCH		ds 1		; Knight SwitchDraw Start
KEND		ds 1		; Knight SwitchDraw End
KPTR		ds 2		; Sprite Pointer
KTYPE		ds 1		; Movement: 00=Static, 01=Follow, 10=Jumping 
KMOVE		ds 1		; Knight Movement Vector
KSHIFT		ds 1		; Jump Shift	
KRESIZE		ds 1		; Resize Line
KNUSIZ0		ds 1		; Resize1
KNUSIZ1		ds 1		; Resize2
KDEL		ds 1		; Delay Counter
			
; Missile Variables
MISSILE		ds #MAXROW	; Missile Positions (0 = No Missile)
MSHAPE		ds 1		; Missile Shape (Pointer LSB)
MPTR		ds 2		; Missile Sprite Pointer
MCOUNT		ds 1		; Current Number of Missiles
MMAX		ds 1		; Maximum Number of Missiles
MDIR		ds 1		; Missile Direction
MDEL		ds 1		; Missile Delay	Value
MTIME		ds 1		; Missile Delay Counter
	
; Bell Variables
BELL		ds #MAXROW	; Bell Positions (0 = No Bell)
BPTR		ds 2		; Bell Sprite Pointer
BELLF		ds 1		; Bell Animation Frame
BELLS		ds 1		; Remaining Bells
					
EndVars                 
        ; Display Remaining RAM
        echo "----",($100 - *) , "bytes left (RAM)" 

; Begin code segment (4K ROM)
        SEG CODE
        ORG $F000

; -----------------------------------------------------------------------------
; Game Initialisation
; -----------------------------------------------------------------------------
        
Start
        ; Wipe Registers 
        CLEAN_START

	; Reset Game Variables
        jsr ResetGame
			
MainLoop
        ; Do Vertical Sync (VSync Routine by Manuel Polik)
        lda #2                  ; VSYNC enable
        sta WSYNC 
        sta VSYNC 
        sta WSYNC  
        sta WSYNC 
        lsr                     ; VSYNC disable
        sta WSYNC  
        sta VSYNC
	
        ; Set Vertical Blank Timer
        IF PALTIME
        ldy #53                 ; (45*76)/64 = 53
        ELSE
        ldy #43                 ; (37*76)/64 = 43
        ENDIF
        sty TIM64T

; -----------------------------------------------------------------------------
; Main Game Logic (Vertical Blank)
; -----------------------------------------------------------------------------
        
        ; Calculate Sprite Positions & Game Data (Vertical Blank)
GameLogic
        ; Update Game Cycle
        dec CYCLE
	bne EndTime
	
	; Decrease Level Timer (Decrease Speed Later)
	lda LTIME
	and #%11100000
	tay
	lda LTIME
	and #%00011111
	sec
	sbc #1
	bpl EndReset
	lda #15
EndReset
	sta LTIME
	tya
	ora LTIME
	sta LTIME
EndTime

	; Check If Player Is Dead
	lda COLLIDE
	and #%00000001
	beq MoveJoy
	; Fall Down
	dec PLAYERY
	jmp EndMove
MoveJoy
	; Move Every Second Cycle (Make This Variable Later)
	lda CYCLE
	lsr
	bcs RightJoy
	jmp EndMove
        ; Move Right
RightJoy
	lda #%10000000
	and COLLIDE
	beq LeftJoy
	bit SWCHA
	bne LeftJoy
	; Increment Position
        inc PLAYERX
	; Clear Ladder Bit
	lda #%11111101
	and COLLIDE
	sta COLLIDE
        ; Player Sprite Not Reflected
        lda #0
        sta REFP0
	; Play Sound
	lda #7
        sta AUDF0	
        lda #6
        sta AUDV0	
        asl
        sta AUDC0       
        lda #1
        sta DUR0
	bne DownJoy
	; Move Left
LeftJoy
	lda #%01000000
	and COLLIDE
	beq DownJoy
	bit SWCHA
	bne DownJoy
        ; Decrement Position
        dec PLAYERX
	; Clear Ladder Bit
	lda #%11111101
	and COLLIDE
	sta COLLIDE	
        ; Reflect Sprite
        lda #8
        sta REFP0
	; Play Sound
	lda #7
        sta AUDF0	
        lda #6
        sta AUDV0	
        asl
        sta AUDC0       
        lda #1
        sta DUR0	
	; Move Down Ladder
DownJoy
	lda #%00100000
	and COLLIDE
	beq UpJoy
	bit SWCHA
	bne UpJoy
	dec PLAYERY
	; Play Sound
        lda #6
        sta AUDF0 
	lda #6
        sta AUDV0
        asl
        sta AUDC0	
        lda #1
        sta DUR0
	; Set Ladder Bit
	lda COLLIDE
	ora #%00000010
	sta COLLIDE
	jmp EndJoy
	; Move Up Ladder
UpJoy
	lda #%00010000
	and COLLIDE
	beq EndJoy
	bit SWCHA
	bne EndJoy
	inc PLAYERY
	; Play Sound
        lda #4
        sta AUDF0
	lda #6
        sta AUDV0	
        asl
        sta AUDC0       
        lda #1
        sta DUR0	
	; Set Ladder Bit
	lda COLLIDE
	ora #%00000010
	sta COLLIDE	
EndJoy

JumpJoy
	; Check For Top Collisions
 	lda #%00001000
 	and COLLIDE
 	beq StopJump
        ; Check If Already Jumping
        lda JSTEP
        beq NewJump
	; Play Sound
	sec
	lda #JUMPHEIGHT+4
	sbc JSTEP
        sta AUDF0
	lda #6
        sta AUDV0
        asl
        sta AUDC0	
        lda #1
        sta DUR0	
        ; Increment Jump Step
        inc JSTEP
        ; Check Jump End
	lda JSTEP	
        cmp #JUMPHEIGHT
	beq EndJump		; Pause At Jump Apex
 	bcs StopJump		; Finish Jump
	inc PLAYERY
	jmp EndJump
NewJump
	; Check Jump Allowed
	lda #%00000100
	and COLLIDE
	bne EndJump
	; Read Fire Button
        lda INPT4
        bmi EndJump
StartJump
	inc JSTEP
	; Clear Ladder Bit
	lda #%11111101
	and COLLIDE
	sta COLLIDE
	jmp EndJump
StopJump
	lda #0
	sta JSTEP
EndJump

; OffLadder
; 	; Clear Ladder Bit If We Can't Move Up/Down
; 	lda #%00110000
; 	and COLLIDE
; 	bne EndLadder
; 	lda #%11111101
; 	and COLLIDE
; 	sta COLLIDE
; EndLadder
	
	; Player Fall
FallDown
	; Don't Fall If Jumping
	lda JSTEP
	bne EndFall
	; Check Collision Vector
	lda #%00000100
	and COLLIDE
	beq EndFall
	; Decrease Position
	dec PLAYERY
	; Play Sound
	lda #KERNH+5
	sbc PLAYERY
	lsr
	lsr
        sta AUDF0 
	lda #6
        sta AUDV0
        asl
        sta AUDC0	
        lda #1
        sta DUR0	
	; Clear Ladder Bit
 	lda COLLIDE
 	and #%11111101
 	sta COLLIDE
EndFall
	
EndMove
	; Set Knight Movement Type
	lda KTYPE
	bne ShowKnight
	; Skip Movement If Static
	jmp EndKnight
ShowKnight
	cmp #1
	beq KFollow

	; PATTERN 2 - JUMPING
KJump
        ; Move Knights
	lda CYCLE
	lsr
	bcc EndKJump	
        lda #%00100000
        bit CYCLE
        bne EndKJump    
        lsr
        bit CYCLE
        bne KMoveDown
KMoveUp
        dec KSHIFT
        BITABS 
KMoveDown
        inc KSHIFT
EndKJump	
	jmp EndKnight
	; PATTERN 1 - FOLLOW PLAYER
KFollow
	
	; Knight Fall
KnightFall
	lda CYCLE
	lsr
	bcs EndKFall
	lda #%00010000
	and KMOVE
	beq EndKFall
	dec KNIGHTY
	jmp EndKnight
EndKFall
		
	; Delay Knight by KDEL Frames
	dec KDEL
	bpl EndKnight
	lda #KDELAY
	sta KDEL
	; Don't Move Knight If Falling
  	lda #%00010000
  	and KMOVE
  	bne EndKnight
	; Move Knight Up Ladder
KnightUp
	lda #%00000001
	and KMOVE
	beq KnightDown
	sec
	lda KNIGHTY
	sbc #2
	cmp PLAYERY
	bcs KnightDown
	inc KNIGHTY
	jmp EndKnight
	; Move Knight Down Ladder
KnightDown
	lda #%00000010
	and KMOVE
	beq KnightLeft
	clc
	lda PLAYERY
	adc #2
	cmp KNIGHTY
	bcs KnightLeft
	dec KNIGHTY
	jmp EndKnight
	; Move Knight Left
KnightLeft
	lda #%00000100
	and KMOVE
	beq KnightRight		
	lda PLAYERX
	cmp KNIGHTX
	bcs KnightRight
	dec KNIGHTX
	lda #0
        sta REFP1	
	jmp EndKnight
	; Move Knight Right
KnightRight
	lda #%00001000
	and KMOVE
	beq EndKnight
	lda KNIGHTX
	cmp PLAYERX
	bcs EndKnight
	inc KNIGHTX
	lda #8
        sta REFP1	
EndKnight		

	; Update Bell Animation Frame
	lda CYCLE
	lsr
	lsr
	lsr
	and #%00000111
	tax
	lda BellCycle,X
	sta BELLF

	; Skip Firing If No Missiles
	lda MDIR
	beq EndMissileLoop
        ; Delay Missile Fire (Otherwise All Missiles Synchronised)
        lda #%00011111
        bit CYCLE
        bne EndNewMissile 
	; Check If Maximum Reached
        lda MCOUNT
        cmp MMAX
        bcs EndNewMissile
	; Start New Missiles
NewMissile
	; Get Random Number 1 < RAND < 11
        jsr Random128
	and #%00000111		; Get Number in Range 0->7
	sta END	
	jsr Random128
	and #%00000011		; Add Number in Range 0->3
	clc
	adc END
	sta END
	jsr Random128
	and #%00000001		; Add Number in Range 0->1
	sec
	adc END
        ; Check if Missile Position Already Taken
        tay     
        lda MISSILE,Y
        bne EndNewMissile
        ; Set Missile Direction
	ldx MDIR
RMissile
	cpx #%01
	bne LMissile
	lda #MISSILEMAX2
	bne StoreNewMissile
LMissile
	cpx #%10
	bne BMissile
        lda #MISSILEMIN
	bne StoreNewMissile
BMissile
        lda #MISSILEMIN	
        bit RANDOM
        bpl StoreNewMissile
        lda #MISSILEMAX2
StoreNewMissile
        sta MISSILE,Y
        inc MCOUNT
EndNewMissile

	; Delay Missile Movement
	dec MTIME
	bpl EndMissileLoop
	lda MDEL
	sta MTIME	
        ; Move All Missiles
        ldy #MAXROW-1
MissileLoop
        ; Check if missile exists
        ldx MISSILE,Y
        beq NextMissile
        ; Check missile direction
        bmi DecMissile
        ; Increment Position
IncMissile
        inx
        cpx #MISSILEMAX
        bne StoreMissile
        beq ResetMissile
DecMissile
        ; Decrement Position
        dex
        cpx #MISSILEMIN2
        bcs StoreMissile
ResetMissile
        ; Decrement Counter
        dec MCOUNT
	; Reset Position
        ldx #0
StoreMissile
        stx MISSILE,Y
NextMissile
        dey
        bpl MissileLoop
EndMissileLoop
	
	; Set Missile Shape
	ldx MDIR
RShape
	cpx #%01
	bne LShape	
	lda #<Missile2
	jmp EndShape
LShape
	cpx #%10
	bne BShape
	lda #<Missile1
	BITABS
BShape	
	lda #<Missile3
EndShape
	sta MSHAPE
	
	; Reset Sprite Pointers (MSB)
	lda #>StartSprites
	sta PPTR+1
	sta KPTR+1
	sta BPTR+1
	sta MPTR+1

	; Reset Screen Pointers (LSB)
	lda #0
	sta SPTR0
	sta SPTR1
	sta SPTR2
	sta SPTR3
		
        ; Calculate SwitchDraw Start For Player and Knight
        sec
        lda PLAYERY
        sta PSWITCH
        sbc #PHEIGHT
        sta PEND
        clc
        lda KNIGHTY
	adc KSHIFT		; Add Shift Value
        sta KSWITCH
	sec
        sbc #KHEIGHT
        sta KEND
	        	   
        ; Calculate Player Pointer (Animated)
	lda COLLIDE
	and #%00000010		; Check If On Ladder
	beq HorizFrame
VertFrame
	lda PLAYERY
	and #%00000100
	lsr
	lsr
	tax
	lda VertFrames,X
	jmp EndFrame
HorizFrame
        lda PLAYERX
	and #%00001100
	lsr
	lsr
	tax
	lda HorizFrames,X
EndFrame
	sec
        sbc PEND
        sta PPTR
	
	; Calculate Knight Pointer (Animated)
	clc
	lda KNIGHTX
	adc KSHIFT
	and #%00000010
	lsr
	tax
	sec
	lda KnightFrames,X
        sbc KEND
        sta KPTR	

	; Fix-up SwitchDraw End Pointers
	lda PEND
        ora #%10000000
        sta PEND	
	lda KEND
        ora #%10000000
        sta KEND	
	
	; Fix Top Line Corruption (Copy END to SWITCH if at top)
	lda PLAYERY
	cmp #KERNH
	bcc NoPCopy
	lda PEND
	sta PSWITCH
NoPCopy
	lda KNIGHTY
	cmp #KERNH
	bcc NoKCopy
	lda KEND
	sta KSWITCH
NoKCopy
					
        ; Position Player Sprite
        ldx #0
        lda PLAYERX
        jsr XPosition
        ; Position Soldier Sprite
        lda KNIGHTX
        jsr XPosition

        ; Reset Collision Vector
        sta CXCLR	

        ; Commit Initial Sprite Movements
        sta WSYNC
	sta HMOVE

        ; Silence Sounds
        lda DUR0
        bne Dec0
        sta AUDV0
        BITABS                  ; Skip Next Instruction
Dec0
        dec DUR0
	
	; Set Starting Row
	lda #MAXROW-1
	sta ROW	

	; Set PF Score Bit To Display Split Colours
	lda #%00000011
	sta CTRLPF

	; Set Sprite Size
	lda KNUSIZ0
	sta NUSIZ1
		
	jmp WaitVblank
	ALIGN 256

        ; Wait for Vertical Blank End
WaitVblank
        lda INTIM
        bne WaitVblank
        sta VBLANK
	
	; Reset Sprite Movements
        sta HMCLR	

	; Display Lives and Time
	ldy #2			; Set Lines To Repeat	
LivesLoop
        sta WSYNC		; [0]
	lda LTIME		; [0] + 3
	lsr			; [3] + 2
	lsr			; [5] + 2
	lsr			; [7] + 2
	lsr			; [9] + 2
	tax			; [11] + 2 
	lda Lives,X		; [13] + 4
	sta PF1			; [17] + 3	< 28
	lda Lives+1,X		; [20] + 4
	sta PF2			; [24] + 3	< 38
	lda LTIME		; [27] + 3
	and #%00011111		; [30] + 2
	tax			; [32] + 2
	lda TimeBar1,X		; [34] + 4
	sta PF1			; [38] + 3	> 38 < 60
	lda TimeBar2,X		; [41] + 4
	sta PF2			; [45] + 3	= 48 EXACT
	dey			; [48] + 2
	bne LivesLoop		; [50] + 2/3

	; Display Side Walls (Used for masking sprites)
	lda #%11000000		; [52] + 2
	sta.w PF0		; [54] + 4
	
	; Set Display Height
	ldy #KERNH		; [58] + 2
		
	; Set PF Colour
	ldx #0			; [60] + 2
	stx PF2			; [62] + 3	> 60
	lda #%00000101		; [65] + 2
	sta CTRLPF		; [67] + 3	
	stx PF1			; [70] + 3	> 70
		
        ; Draw the Screen (Kernel)
        jmp Kernel1		; [73] + 3
EndScreen

; -----------------------------------------------------------------------------
; Collision Detection (Overscan)
; -----------------------------------------------------------------------------

	; Construct Player Collision Vector COLLIDE
	; Bit 7 - Can Move Right
	; Bit 6 - Can Move Left
	; Bit 5 - Can Move Down Ladder
	; Bit 4 - Can Move Up Ladder
	; Bit 3 - Can Jump
	; Bit 2 - Fall
	; Bit 1 - On Ladder
	; Bit 0 - Dead

	; Reset Collision Vector
	lda COLLIDE
	and #%00000011		; Reset Bits 7-2
	ora #%11001100		; Set Defaults
	sta COLLIDE

	; Check If Dead
	lda COLLIDE
	and #%00000001
	beq CheckRight
	lda PLAYERY
	bpl CheckRight
	; Reset
	jsr ResetGame
	; Check Player RHS
CheckRight
        ; Check Screen Edge
        lda PLAYERX
        cmp #XMAX+1
        bcs RightColl
	; Check Upper Square
	adc #7
	tax
	ldy PLAYERY
	dey
	dey
	dey
	jsr MapCords
	; Wall Collision
	cmp #1
	beq RightColl
	; Check Lower Square
	clc
	lda PLAYERX
	adc #7
	tax
	sec
	lda PLAYERY
	sbc #8
	tay
	jsr MapCords
	; Wall Collision
	cmp #1
	beq RightColl
	jmp EndRight
RightColl
	lda COLLIDE
	and #%01111111
	sta COLLIDE
EndRight

	; Check Player LHS
CheckLeft	
        ; Check Screen Edge
        ldx PLAYERX
        cpx #XMIN-1
        bcc LeftColl
	; Check Upper Square
	ldy PLAYERY
	dey
	dey
	dey
	; ldx PLAYERX (Already Loaded)
	jsr MapCords
	cmp #1
	beq LeftColl
	; Check Lower Square
	ldx PLAYERX
	sec
	lda PLAYERY
	sbc #8
	tay
	jsr MapCords
	cmp #1
	beq LeftColl
	jmp EndLeft
LeftColl
	lda COLLIDE
	and #%10111111
	sta COLLIDE
EndLeft		

CheckDown
	; Check Screen Bottom
	lda PLAYERY
	cmp #PHEIGHT
	bcs OnScreen
	jmp Die
OnScreen
	; Check LHS Square
	sec
	sbc #PHEIGHT-2
	tay
	ldx PLAYERX
	inx
	jsr MapCords
	cmp #1
	beq DownColl
	cmp #0
	beq CheckDown2
	lda COLLIDE
	and #%11011011
	ora #%00100000
	sta COLLIDE
CheckDown2
	; Check RHS Square
	clc
	lda PLAYERX
	adc #6
	tax
	sec
	lda PLAYERY
	sbc #PHEIGHT-2
	tay
	jsr MapCords
	cmp #1
	beq DownColl
	cmp #0
	beq EndDown
	lda COLLIDE
	and #%11011011	
	ora #%00100000
	sta COLLIDE
	jmp EndDown
DownColl
	lda COLLIDE
	and #%11011011
	sta COLLIDE
EndDown	

CheckUp
	; Check Screen Edge
	ldy PLAYERY
	cpy #KERNH
	bcs UpColl
	; Check LHS Square
	dey
	dey
	; clc (Already Clear)
	lda PLAYERX
	adc #6
	tax
	jsr MapCords
	cmp #1
	beq UpColl
	cmp #2
	bne CheckUp2
	lda COLLIDE
	ora #%00010000
	sta COLLIDE
CheckUp2
	; Check RHS Square
	ldx PLAYERX
	inx
	ldy PLAYERY
	dey
	dey
	jsr MapCords
	cmp #1
	beq UpColl
	cmp #2
	bne EndUp
	lda COLLIDE
	ora #%00010000
	sta COLLIDE	
	jmp EndUp
UpColl
	lda COLLIDE
	and #%11100111
	sta COLLIDE
EndUp

	; Construct Knight Collision Vector KMOVE
	; Bit 7-5 - Reserved
	; Bit 4 - Fall 
	; Bit 3 - Can Move Right
	; Bit 2 - Can Move Left
	; Bit 1 - Can Move Down Ladder
	; Bit 0 - Can Move Up Ladder
	lda #%00011111
	sta KMOVE
			
	; Check Knight RHS
CheckKRight
	; Check Screen Edge
	lda KNIGHTX
        cmp #XMAX
        bcs KRightColl
        ; Check Upper Square
        adc #8
        tax
        ldy KNIGHTY
	dey
	dey
	dey
        jsr MapCords
        ; Wall Collision
        cmp #1
        beq KRightColl
        ; Check Lower Square
        clc
        lda KNIGHTX
        adc #8
        tax
        sec
        lda KNIGHTY
        sbc #10
        tay
        jsr MapCords
        ; Wall Collision
        cmp #1
        beq KRightColl
	jmp EndKRight
KRightColl
	lda KMOVE
	and #%11110111
	sta KMOVE
EndKRight
	
	; Check Knight LHS
CheckKLeft	
        ; Check Screen Edge
        ldx KNIGHTX
        cpx #XMIN
        bcc KLeftColl
	; Check Upper Square
	dex
	ldy KNIGHTY
	dey
	dey
	dey
	jsr MapCords
	cmp #1
	beq KLeftColl
	; Check Lower Square
	ldx KNIGHTX
	dex
	sec
	lda KNIGHTY
	sbc #10
	tay
	jsr MapCords
	cmp #1
	beq KLeftColl
	jmp EndKLeft
KLeftColl
	lda KMOVE
	and #%11111011
	sta KMOVE
EndKLeft

CheckKDown	
	; NEED TO CHECK SCREEN BOTTOM!
	; Check LHS Square
	ldx KNIGHTX
	sec
	lda KNIGHTY
	sbc #11
	tay
	jsr MapCords
	cmp #1
	beq KDownColl
	cmp #2
	bne CheckKDown2
	lda KMOVE
	and #%11101111
	sta KMOVE
CheckKDown2
	; Check RHS Square
	clc
	lda KNIGHTX
	adc #7
	tax
	sec
	lda KNIGHTY
	sbc #11
	tay
	jsr MapCords
	cmp #1
	beq KDownColl
	cmp #2
	bne EndKDown
	lda KMOVE
	and #%11101111
	sta KMOVE
	jmp EndKDown
KDownColl
	lda KMOVE
	and #%11101101
	sta KMOVE
EndKDown

CheckKUp
	; Check Screen Edge
	ldy KNIGHTY
	cpy #KERNH
	bcs KUpColl
	; Check LHS Square
	dey
	dey
	clc
	lda KNIGHTX
	adc #7
	tax
	jsr MapCords
	cmp #2
	bne KUpColl
	; Check RHS Square
	ldx KNIGHTX
	ldy KNIGHTY
	dey
	dey
	jsr MapCords
	cmp #2
	bne KUpColl
	jmp EndKUp
KUpColl
	lda KMOVE
	and #%11111110
	sta KMOVE	
EndKUp
	
	; Check Bell Collisions
	lda CXM0P
	and #%01000000
	beq EndBell
	; Calculate Player Width
	clc
	lda PLAYERX
	adc #6
	sta PEND
	; Check First Player Row
CheckBell1
	; Calculate Upper Row (PLAYERY-3/8)
	sec
	lda PLAYERY
	sbc #1
	lsr
	lsr
	lsr
	tay			; ROW
	; Fetch Bell Data For Row
	lda BELL,Y
	; Move to Next Row If No Bell
	beq BellColl2
	; Convert Position To Player Coordinates (POS + 7)
	clc
	adc #7
	sta PSWITCH		; BELLX
Check1
	; Check If BELLX Is Between PLAYERX and PLAYERX+5
	cmp PLAYERX
	bmi Check2
	cmp PEND
	bmi BellColl1
Check2
	; Calculate Bell Width	
	clc
	adc #8
	sta PEND		; BELLX+8
	; Check If PLAYERX Is Between BELLX and BELLX+7
	lda PLAYERX
	cmp PSWITCH
	bmi BellColl2
	cmp PEND
	bpl BellColl2
BellColl1
	; Remove Bell
	lda #0
	sta BELL,Y
	dec BELLS
	jmp EndBell
BellColl2
	; Check Lower Row (PLAYERY-8/8)
	sec
	lda PLAYERY
	sbc #8
	lsr
	lsr
	lsr
	tay			; ROW
	; Check That Bell Exists (Flag Error Otherwise)
	lda BELL,Y
	beq BellError
	; Remove Bell
	lda #0
	sta BELL,Y
	dec BELLS
	jmp EndBell
BellError
	; Debugging 
	lda #$06
	sta COLUBK
EndBell

	; Check Console Switches
	lda SWCHB
	cmp SWCH
	beq EndSwitches
	sta SWCH
	; Select Changes Level (Debugging)
	lda #%00000010
	bit SWCH
	bne EndSwitches
	inc LEVEL
	jsr NextLevel
EndSwitches

	; Check Bell Numbers
	lda BELLS
	bne EndBellCheck
	; Move To Next Level
	inc LEVEL
	jsr NextLevel
EndBellCheck
	
	; Check Player/Missile and Player/Knight Collisions
	lda CXM1P
	bmi Die
	lda CXPPMM
	bmi Die
	jmp EndCollision
Die
	; Set Dead Bit
	lda #%00000001
	sta COLLIDE
EndCollision
	
        ; Finish Overscan
WaitOverscanEnd
        lda INTIM
        bne WaitOverscanEnd
        
        ; Loop To Beginning
        jmp MainLoop
	
	; MapCords Subroutine
	; Translates Coordinates (X,Y) to Map
	; On Entry - X,Y = Coordinates
	; On Exit - A = Map Data 
MapCords
	; Calculate Row Y/8 (0-11)
	tya			; [0] + 2
	lsr			; [2] + 2
	and #%11111100		; [4] + 2
	sta ROW			; [6] + 3	ROW
	; Calculate Column (X-XMIN)/8 (0-15)
	sec			; [9] + 2
	txa			; [11] + 2
	sbc #XMIN-1		; [13] + 2
	lsr			; [15] + 2
	lsr			; [17] + 2
	lsr			; [19] + 2
	tax			; [21] + 2	COL
	; Calculate Block COL/4 (0-3)
	lsr			; [23] + 2
	lsr			; [25] + 2	BLOCK
	; Calculate Screen Index (ROW*4) + BLOCK
	clc			; [27] + 2
	adc ROW			; [29] + 3
	tay			; [32] + 2	INDEX
	; Make Pointer and Fetch Map Data
	lda SCREEN,Y		; [34] + 4
	sta PPTR		; [38] + 3
	lda BBASE		; [41] + 3
	sta PPTR+1		; [44] + 3
	ldy #0			; [47] + 2
	lda (PPTR),Y		; [49] + 5
	; Mask and Shift Data			
	and Masks,X		; [54] + 3	; MASK
	ldy Shifts,X		; [57] + 3	; SHIFT
	beq EndShift		; [60] + 2/3
ShiftLoop			
	lsr			; [62] + 2
	lsr			; [64] + 2
	dey			; [66] + 2
	bne ShiftLoop		; [68] + 2/3
EndShift
	rts
			
; -----------------------------------------------------------------------------
; Display Kernel
; -----------------------------------------------------------------------------

	ALIGN 256

	; Player SwitchDraw Routines
PSwitch1
        lda PEND                ; [6] + 3      
        sta.w PSWITCH           ; [9] + 4
        bne PContinue1          ; [13] + 3
PWait1
        SLEEP 3                 ; [8] + 3
	lda #0			; [11] + 2
        beq PContinue1          ; [13] + 3
	
	; Knight SwitchDraw Routines
KSwitch1
        lda KEND                ; [63] + 3      
        sta KSWITCH             ; [66] + 3
        bne KContinue1          ; [69] + 3
KWait1
        nop			; [65] + 2
	lda #0			; [67] + 2
        beq KContinue1          ; [69] + 3
			
; LINE 0 - REPOSITION BELL
Kernel1
	; Disable Bell
	stx ENAM0		; [0] + 3
	
	; Fetch Bell Position
	ldx ROW			; [3] + 2
	lda BELL,X		; [5] + 4
        ; Divide Position by 15
	sec			; [9] + 2
BDiv15
        sbc #15                 ; [11] + 2
        bcs BDiv15              ; [13/18/23/28/33/38/43/48/53] + 2/3
        ; Store Position
	tax			; [55] + 2
        lda FineTuneEnd,X       ; [57] + 5*
	sta.w RESM0		; [62] + 3		
        sta HMM0                ; [65] + 3
	
	; Decrease Line Counter
	dey			; [68] + 2		
			
; LINE 1 - DISPLAY PLAYER
	sta WSYNC		; [0]
	
	; Draw Player
	cpy PSWITCH             ; [0] + 3
        beq PSwitch1            ; [3] + 2/3
        bpl PWait1              ; [5] + 2/3
        lda (PPTR),Y            ; [7] + 5
        sta.w GRP0		; [12] + 4
PContinue1

	; Draw Screen
	lda (SPTR0),Y		; [16] + 5
	sta PF1			; [21] + 3	< 28
	lda (SPTR1),Y		; [24] + 5	
        sta PF2                 ; [29] + 3	< 38
	lda (SPTR3),Y		; [32] + 5
	sta PF1			; [37] + 3      > 38 < 60
	lda (SPTR2),Y		; [40] + 5
	sta PF2			; [45] + 3      = 48 EXACT

	; Calculate Wall Position
	sec			; [48] + 2
        tya			; [50] + 2
        sbc #WHEIGHT+1		; [52] + 2
        sta END			; [54] + 3
	
	; Draw Knight
	cpy KSWITCH             ; [57] + 3
        beq KSwitch1            ; [60] + 2/3
        bpl KWait1              ; [62] + 2/3
        lda (KPTR),Y            ; [64] + 5
        sta GRP1                ; [69] + 3
KContinue1

	; Fetch Display Row
	lda ROW			; [72] + 3
	
	; Commit BellMovement
	sta HMOVE		; [75] + 3
	
	; Calculate Screen Offset (ROW * 4)
	asl			; [2] + 2	
	asl			; [4] + 2
	tax			; [6] + 2
	
; LINE 2 - UPDATE SCREEN POINTERS
	
	; Draw Screen
	lda (SPTR0),Y		; [8] + 5
	sta PF1			; [13] + 3	< 28
	lda (SPTR1),Y		; [16] + 5	
        sta PF2                 ; [21] + 3	< 38

	; Calculate SPTR0 Pointer
        sec			; [24] + 2
        lda SCREEN,X		; [26] + 4
        sbc END			; [30] + 3
        sta SPTR0		; [33] + 3

	nop			; [36] + 2
	sec			; [38] + 2
	
	lda (SPTR2),Y		; [40] + 5
	sta PF2			; [45] + 3      = 48 EXACT
	lda (SPTR3),Y		; [48] + 5
	sta PF1			; [53] + 3      > 38 < 60

	; Calculate SPTR1 Pointer
        lda SCREEN+1,X		; [56] + 4
        sbc END			; [60] + 3
        sta SPTR1		; [63] + 3

	; Calculate SPTR2 Pointer
        sec			; [66] + 2
        lda SCREEN+2,X		; [68] + 4
        sbc END			; [72] + 3
        sta SPTR2		; [75] + 3
		
	; Decrease Line Counter
	dey			; [2] + 2
	
; LINE 3 - DISPLAY PLAYER & CALCULATE SCREEN	
	; Draw Player
	cpy PSWITCH             ; [4] + 3
        beq PSwitch2            ; [7] + 2/3
        bpl PWait2              ; [9] + 2/3
        lda (PPTR),Y            ; [11] + 5
        sta GRP0		; [16] + 3
PContinue2

	; Clear PF
	lda #0			; [19] + 2
	sta PF1			; [21] + 3	< 28
	sta PF2			; [24] + 3	< 38
			
	; Calculate SPTR3 Pointer
	sec			; [27] + 2
        lda SCREEN+3,X		; [29] + 4
        sbc END			; [33] + 3
        sta SPTR3		; [36] + 3

	; Calculate Bell Pointer
        lda BELLF		; [39] + 3
        sbc END			; [42] + 3
        sta BPTR		; [45] + 3

	; Calculate Missile Pointer
	ldx ROW			; [48] + 3
	lda MSHAPE		; [51] + 3
        sbc END			; [54] + 3
        sta MPTR		; [57] + 3
	
	; Draw Knight
	cpy KSWITCH             ; [60] + 3
        beq KSwitch2            ; [63] + 2/3
        bpl KWait2              ; [65] + 2/3
        lda (KPTR),Y            ; [67] + 5
        sta GRP1                ; [72] + 3
KContinue2
	
	; Clear Sprite Movements
	sta HMCLR		; [75] + 3

	; Decrease Line Counter
	dey			; [2] + 2
		
; LINE 4 - REPOSITION MISSILE
		
	; Fetch Bell Position
	lda MISSILE,X		; [4] + 4
	asl			; [8] + 2
        ; Divide Position by 15
MDiv15
        sbc #15                 ; [10] + 2
        bcs MDiv15              ; [12/17/22/27/32/37/42/47/52] + 2/3
        ; Get Fine Position
        tax                     ; [54] + 2
        lda FineTuneEnd,X       ; [56] + 5
	; Set MainKernel Counter
	ldx #6			; [61] + 2	
	; Store Coarse Position
        sta RESM1		; [63] + 3	
        sta HMM1                ; [66] + 3
	jmp Kernel2		; [69] + 3
	
	; Player SwitchDraw Routines
PSwitch2
        lda PEND                ; [10] + 3      
        sta PSWITCH		; [13] + 3
        bne PContinue2          ; [16] + 3
PWait2
	nop			; [12] + 2
        lda #0			; [14] + 2
        beq PContinue2          ; [16] + 3

	; Knight SwitchDraw Routines
KSwitch2
        lda KEND                ; [63] + 3      
        sta KSWITCH             ; [66] + 3
        bne KContinue2          ; [69] + 3
KWait2
        nop			; [65] + 2
	lda #0			; [67] + 2
        beq KContinue2          ; [69] + 3

EndKernel1
	if (>Kernel1 != >EndKernel1)
          echo "WARNING: Kernel 1 crosses a page boundary!"
        endif	
	
	ALIGN 256

	; Player SwitchDraw
PSwitch3
        lda PEND                ; [6] + 3      
        sta.w PSWITCH		; [9] + 4
        bne PContinue3          ; [13] + 3
PWait3
        SLEEP 3                 ; [8] + 3
	lda #0			; [11] + 2
        beq PContinue3          ; [13] + 3
	
	; Knight SwitchDraw
KSwitch3
        lda KEND                ; [54] + 3      
        sta KSWITCH             ; [57] + 3
        bne KContinue3          ; [60] + 3
KWait3
        nop			; [56] + 2
	lda #0			; [58] + 2
	beq KContinue3          ; [60] + 3	
	
Kernel2
	sta WSYNC		; [0]
	
; LINE 5-15 DISPLAY SPRITES & SCREEN (MAIN KERNEL LOOP)
KernelLoop
	; Draw Player
	cpy PSWITCH             ; [0] + 3
        beq PSwitch3            ; [3] + 2/3
        bpl PWait3              ; [5] + 2/3
        lda (PPTR),Y            ; [7] + 5
        sta.w GRP0		; [12] + 4
PContinue3
		
	; Draw Screen
	lda (SPTR0),Y		; [16] + 5
	sta PF1			; [21] + 3	> 70 < 28
	lda (SPTR1),Y		; [24] + 5	
        sta PF2                 ; [29] + 3      > 60 < 38
	lda (SPTR3),Y		; [32] + 5
	sta PF1			; [37] + 3      > 38 < 60
	lda (SPTR2),Y		; [40] + 5
	sta PF2			; [45] + 3      = 48 EXACT
		
	; Draw Knight
	cpy KSWITCH             ; [48] + 3
        beq KSwitch3            ; [51] + 2/3
        bpl KWait3              ; [53] + 2/3
        lda (KPTR),Y            ; [55] + 5
        sta GRP1		; [60] + 3
KContinue3

	; Check Row
	dex			; [63] + 2
	beq NewLine		; [65] + 2/3
			
	; Load Bell Data
 	lda (BPTR),Y		; [67] + 5
	sta HMM0		; [72] + 3
				
	; Commit Movements
	sta HMOVE		; [75] + 3
	SLEEP 4			; [2] + 4
		
	; Draw Bell (M0)	
	sta ENAM0		; [6] + 3	
 	asl			; [9] + 2
 	asl			; [11] + 2
	sta NUSIZ0		; [13] + 3
	
	; Draw Screen
 	lda (SPTR0),Y		; [16] + 5
 	sta PF1			; [21] + 3	< 28
 	lda (SPTR1),Y		; [24] + 5	
        sta PF2                 ; [29] + 3      < 38
 	lda (SPTR3),Y		; [32] + 5
 	sta PF1			; [37] + 3      > 38 < 60
 	lda (SPTR2),Y		; [40] + 5
 	sta PF2			; [45] + 3      = 48 EXACT
	SLEEP 3
	
	; Draw Missiles/Knights
	cpy KRESIZE		; [51] + 3
	bpl DrawMissile		; [54] + 2/3
	lda KNUSIZ1		; [56] + 3
	sta NUSIZ1		; [59] + 3
	lda #0			; [62] + 2
	SLEEP 4			; [64] + 4
EndResize
	sta ENAM1		; [68] + 3
	
	; Check End Condition
	dey			; [71] + 2
	bne KernelLoop		; [73] + 2/3

	; Clear Screen Variables
	sty PF0
	sty PF1
	sty PF2
	sty ENAM0
	sty ENAM1
	sty ENABL
	sty GRP0
	sty GRP1
	sty HMCLR	
	                                       
        ; Start Vertical Blank
        lda #2
        sta WSYNC
        sta VBLANK

        ; Set Timer for Overscan
        IF PALTIME
        ldy #42                 ; (36*76)/64 = 42
        ELSE
        ldy #35                 ; (30*76)/64 = 35
        ENDIF
        sty TIM64T

        jmp EndScreen
NewLine
	; Decrement Row And Start Again
	dec ROW			; [68] + 5
	jmp Kernel1		; [73] + 3

DrawMissile
	; Display Missile
 	lda (MPTR),Y		; [57] + 5
 	sta HMM1		; [62] + 3
	jmp EndResize		; [65] + 3
	
EndKernel2
        if (>Kernel2 != >EndKernel2)
          echo "WARNING: Kernel 2 crosses a page boundary!"
        endif
		
; -----------------------------------------------------------------------------
; Game Subroutines
; -----------------------------------------------------------------------------

        ; Random Number Generation Routine
        ; On Exit: A = Random Number
Random128
        lda RANDOM
        asl                     
        bcc NoEor  
        eor #$CF
NoEor
        sta RANDOM   
        rts	

        
        ; Sprite Horizontal Positioning Routine (Battlezone Variant)
        ; On Entry: A = Coordinate and X = Object ID
XPosition
        sta WSYNC               ; [0]
        sec                     ; [0] + 2
        ; Divide Position by 15
Divide15
        sbc #15                 ; [2] + 2
        bcs Divide15            ; [4] + 2/3
        nop                     ; [6] + 2
        tay                     ; [8] + 2
        ; Store Coarse and Fine Position
        lda FineTuneEnd,Y       ; [10] + 5*
        sta HMP0,X              ; [14] + 4
        sta RESP0,X             ; [18] + 4
        ; Position At Cycles: [22/27/32/37/42/47/52/57/62/67/72/77]
        inx
        rts

	; Reset Game Routine
ResetGame
	; Reset Lives and Level Time
	lda #%01101111
	sta LTIME	
NextLevel
	; Reset Level Timer (Preserve Lives)
	lda #%11110000
	and LTIME
	ora #%00001111
	sta LTIME
	
	; Construct Pointer To Level
	lda LEVEL
	and #%00000011		; Wrap at Level 4 (for now)
	asl
	tax
	lda LevelPtr,X		; Load LSB
	sta LPTR
	lda LevelPtr+1,X	; Load MSB
	sta LPTR+1
	
	; Construct Screen Data Pointer 
	ldy #0
	lda (LPTR),Y
	sta PPTR
	iny
	lda (LPTR),Y
	sta PPTR+1

	; Copy Screen Data
	ldy #0
ScreenCopyLoop
	lda (PPTR),Y
	sta SCREEN,Y
	iny
	cpy #MAXROW*4
	bcc ScreenCopyLoop

	; Set Block Pointers (MSB)
	ldy #2
	lda (LPTR),Y
	sta BBASE
	sta SPTR0+1
	sta SPTR1+1
	sta SPTR2+1
	sta SPTR3+1

	; Set Block Colour
	iny
	lda (LPTR),Y
	sta COLUPF
	iny

	; Copy Bell Positions & Reset Missile Positions
	ldx #0
BellLoop
	lda (LPTR),Y
	asl
	asl
	asl
	sta BELL,X
	lda #0
	sta MISSILE,X
	iny
	inx
	cpx #MAXROW
	bcc BellLoop

	; Set Total Number Of Bells
	lda (LPTR),Y
	sta BELLS

	; Set Missile Variables
	iny
	lda (LPTR),Y
	sta MDEL
	iny
	lda (LPTR),Y
	sta MDIR
	iny
	lda (LPTR),Y
	sta MMAX

	; Set Sprite Resize Variables
	iny	
	lda (LPTR),Y
	sta KNUSIZ0	
	iny	
	lda (LPTR),Y
	sta KNUSIZ1
	iny	
	lda (LPTR),Y
	sta KRESIZE

	; Set Player Position
	iny	
	lda (LPTR),Y
	sta PLAYERX
	iny	
	lda (LPTR),Y
	sta PLAYERY

	; Set Knight Variables
	iny	
	lda (LPTR),Y
	sta KNIGHTX
	iny	
	lda (LPTR),Y
	sta KNIGHTY
	iny	
	lda (LPTR),Y
	sta KTYPE
	iny	
	lda (LPTR),Y
	sta KDEL		
	
	; Reset Various Variables
	lda #0
	sta COLLIDE
	sta MCOUNT
	sta JSTEP
	sta KSHIFT
	sta CYCLE
	
	; Reset Random Seed (Improve Later?)
	lda #31
	sta RANDOM

	; Set Sprite Colours
	lda #YELL
	sta COLUP0
	lda #BLUE
	sta COLUP1	

	; Set Reflected PF
	lda #%00000101
	sta CTRLPF	
	
	rts	
			
; -----------------------------------------------------------------------------
; Screen & Sprite Data
; -----------------------------------------------------------------------------

	ALIGN 256

LevelData
	; Level Data Pointers
LevelPtr
	DC.B	<Level1,>Level1
	DC.B	<Level2,>Level2
	DC.B	<Level3,>Level3
	DC.B	<Level4,>Level4
	
Level1
; 	ScreenBase
	DC.B	<Screen1,>Screen1
; 	BlockBase
	DC.B	>Block0L
; 	BlockColour
	DC.B	GREEN
; 	BellPositions
	DC.B	0,1,15,16,0,0,14,4,11,15,2,11	
; 	BellTotal
	DC.B	9
; 	MissileDelay (0=None)
	DC.B	3
; 	MissileDir (00=None, 01=RHS, 10=LHS, 11=BiDirectional)
	DC.B	%11
; 	MaxMissiles
	DC.B	3
; 	NUSize1
	DC.B	%00100000
; 	NUSize2
	DC.B	%00100000	
; 	SwapRow
	DC.B	0
; 	PlayerStartX
	DC.B	40
; 	PlayerStartY
	DC.B	40
; 	KnightStartX
	DC.B	50
; 	KnightStartY
	DC.B	18
; 	KnightMovePattern (00=Static, 01=Follow, 10=Jumping, 11=?)
	DC.B	%01
; 	KnightDelay
	DC.B	2
	
Level2
; 	ScreenBase
	DC.B	<Screen2,>Screen2
; 	BlockBase
	DC.B	>Block0L
; 	BlockColour
	DC.B	ORANGE
; 	BellPositions
	DC.B	0,1,13,4,7,9,1,14,16,8,11,1	
; 	BellTotal
	DC.B	11
; 	MissileDelay (0=VeryFast, 1=Fast, 2=Medium, 3=Slow, 4=Crawl)
	DC.B	2
; 	MissileDir (00=None, 01=RHS, 10=LHS, 11=BiDirectional)
	DC.B	%11
; 	MaxMissiles
	DC.B	6
; 	NUSize1
	DC.B	%00100000
; 	NUSize2
	DC.B	%00100110	
; 	SwapRow
	DC.B	0
; 	PlayerStartX
	DC.B	40
; 	PlayerStartY
	DC.B	16
; 	KnightStartX
	DC.B	20
; 	KnightStartY
	DC.B	0
; 	KnightMovePattern (00=Static, 01=Follow, 10=Jumping, 11=Cycle?)
	DC.B	%00
; 	KnightDelay
	DC.B	0
	
Level3
; 	ScreenBase
	DC.B	<Screen3,>Screen3
; 	BlockBase
	DC.B	>Block0L
; 	BlockColour
	DC.B	PURPLE
; 	BellPositions
	DC.B	0,15,1,9,11,16,9,13,1,9,16,6
; 	BellTotal
	DC.B	11
; 	MissileDelay (0=VeryFast, 1=Fast, 2=Medium, 3=Slow, 4=Crawl)
	DC.B	2
; 	MissileDir (00=None, 01=RHS, 10=LHS, 11=BiDirectional)
	DC.B	%10
; 	MaxMissiles
	DC.B	2
; 	NUSize1
	DC.B	%00100000
; 	NUSize2
	DC.B	%00100110	
; 	SwapRow
	DC.B	0
; 	PlayerStartX
	DC.B	40
; 	PlayerStartY
	DC.B	32
; 	KnightStartX
	DC.B	72
; 	KnightStartY
	DC.B	42
; 	KnightMovePattern (00=Static, 01=Follow, 10=Jumping, 11=Cycle?)
	DC.B	%10
; 	KnightDelay
	DC.B	2	

Level4
; 	ScreenBase
	DC.B	<Screen4,>Screen4
; 	BlockBase
	DC.B	>Block0L
; 	BlockColour
	DC.B	RED
; 	BellPositions
	DC.B	0,16,9,1,13,7,16,7,5,16,9,1	
; 	BellTotal
	DC.B	11
; 	MissileDelay (0=VeryFast, 1=Fast, 2=Medium, 3=Slow, 4=Crawl)
	DC.B	3
; 	MissileDir (00=None, 01=RHS, 10=LHS, 11=BiDirectional)
	DC.B	%01
; 	MaxMissiles
	DC.B	2
; 	NUSize1
	DC.B	%00100000
; 	NUSize2
	DC.B	%00100110	
; 	SwapRow
	DC.B	28
; 	PlayerStartX
	DC.B	40
; 	PlayerStartY
	DC.B	56
; 	KnightStartX
	DC.B	56
; 	KnightStartY
	DC.B	18
; 	KnightMovePattern (00=Static, 01=Follow, 10=Jumping, 11=Cycle?)
	DC.B	%10
; 	KnightDelay
	DC.B	2	

EndLevelData
        if (>LevelData != >EndLevelData)
          echo "WARNING: Level Data crosses a page boundary!"
        endif	
				
	ALIGN 256	
ScreenData
	; Screen 1 Data
Screen1
	DC.B	<Block1L,<Block1R,<Block1L,<Block1R
	DC.B	<Block0L,<Block3R,<Block0L,<Block0R	
	DC.B	<Block0L,<Block3R,<Block0L,<Block5R
	DC.B	<Block8L,<Block2R,<Block1L,<Block7R
	DC.B	<Block0L,<Block3R,<Block0L,<Block0R
	DC.B	<Block0L,<Block3R,<Block0L,<Block5R
	DC.B	<Block6L,<Block2R,<Block4L,<Block0R
	DC.B	<Block0L,<Block3R,<Block6L,<Block0R
	DC.B	<Block0L,<Block3R,<Block0L,<Block7R			
	DC.B	<Block4L,<Block2R,<Block9L,<Block0R
	DC.B	<Block0L,<Block3R,<Block0L,<Block5R
	DC.B	<Block0L,<Block0R,<Block0L,<Block0R
Screen2
	DC.B	<Block1L,<Block1R,<Block1L,<Block1R	
	DC.B	<Block0L,<Block0R,<Block0L,<Block3R
	DC.B	<Block0L,<Block0R,<Block0L,<Block3R
	DC.B	<Block7L,<Block1R,<Block1L,<Block2R
	DC.B	<Block3L,<Block5R,<Block3L,<Block3R
	DC.B	<Block3L,<Block5R,<Block3L,<Block0R
	DC.B	<Block0L,<Block1R,<Block2L,<Block7R	
	DC.B	<Block0L,<Block0R,<Block3L,<Block0R
	DC.B	<Block5L,<Block3R,<Block5L,<Block9R
	DC.B	<Block3L,<Block3R,<Block9L,<Block4R
	DC.B	<Block2L,<Block3R,<Block4L,<Block0R
	DC.B	<Block0L,<Block0R,<Block0L,<Block0R
Screen3
	DC.B	<Block7L,<Block2R,<Block0L,<Block4R
	DC.B	<Block0L,<Block3R,<Block0L,<Block0R
	DC.B	<Block5L,<Block0R,<Block0L,<Block3R
	DC.B	<Block3L,<Block5R,<Block3L,<Block3R
	DC.B	<Block4L,<Block0R,<Block0L,<Block0R
	DC.B	<Block3L,<Block0R,<Block3L,<Block0R
	DC.B	<Block5L,<Block3R,<Block0L,<Block8R
	DC.B	<Block0L,<Block0R,<Block3L,<Block4R
	DC.B	<Block4L,<Block0R,<Block0L,<Block4R
	DC.B	<Block0L,<Block3R,<Block3L,<Block0R	
	DC.B	<Block5L,<Block2R,<Block2L,<Block0R
	DC.B	<Block0L,<Block0R,<Block0L,<Block0R			
Screen4
	DC.B	<Block5L,<Block8R,<Block4L,<Block4R
	DC.B	<Block0L,<Block7R,<Block7L,<Block0R
	DC.B	<Block3L,<Block0R,<Block0L,<Block0R
	DC.B	<Block3L,<Block0R,<Block3L,<Block0R	
	DC.B	<Block3L,<Block6R,<Block2L,<Block4R
	DC.B	<Block2L,<Block0R,<Block0L,<Block5R	
	DC.B	<Block0L,<Block8R,<Block1L,<Block4R
	DC.B	<Block4L,<Block4R,<Block3L,<Block0R		
	DC.B	<Block4L,<Block4R,<Block0L,<Block5R
	DC.B	<Block4L,<Block1R,<Block2L,<Block0R	
	DC.B	<Block0L,<Block0R,<Block0L,<Block0R
	DC.B	<Block0L,<Block7R,<Block7L,<Block5R

EndScreenData
        if (>ScreenData != >EndScreenData)
          echo "WARNING: Screen Data crosses a page boundary!"
        endif	
	
	ALIGN 256	
		
	; Lives PF Data
Lives	DC.B	%01000000	; 1 Life
	DC.B	%00000000
	DC.B	%01010000	; 2 Lives
	DC.B	%00000000
	DC.B	%01010100	; 3 Lives
	DC.B	%00000000
	DC.B	%01010101	; 4 Lives
	DC.B	%00000000
	DC.B	%01010101	; 5 Lives
	DC.B	%00000010
	DC.B	%01010101	; 6 Lives
	DC.B	%00001010
	DC.B	%01010101	; 7 Lives
	DC.B	%00101010
	DC.B	%01010101	; 8 Lives
	DC.B	%10101010	

	; Time Bar PF Data
TimeBar1						
	DC.B	%00000000	; 0
	DC.B	%01000000	; 1
	DC.B	%01100000	; 2
	DC.B	%01110000	; 3
	DC.B	%01111000	; 4
	DC.B	%01111100	; 5
	DC.B	%01111110	; 6
	DC.B	%01111111	; 7
	DC.B	%01111111	; 8
	DC.B	%01111111	; 9
	DC.B	%01111111	; 10
	DC.B	%01111111	; 11
	DC.B	%01111111	; 12
	DC.B	%01111111	; 13
	DC.B	%01111111	; 14
	DC.B	%01111111	; 15
TimeBar2
	DC.B	%00000000	; 0
	DC.B	%00000000	; 1
	DC.B	%00000000	; 2
	DC.B	%00000000	; 3
	DC.B	%00000000	; 4
	DC.B	%00000000	; 5
	DC.B	%00000000	; 6
	DC.B	%00000000	; 7
	DC.B	%00000000	; 8
	DC.B	%00000001	; 9
	DC.B	%00000011	; 10
	DC.B	%00000111	; 11
	DC.B	%00001111	; 12
	DC.B	%00011111	; 13
	DC.B	%00111111	; 14
	DC.B	%01111111	; 15
	
	; Collision Detection Masks and Shift (For Map Data)
Masks
	DC.B	%11000000
	DC.B	%00110000
	DC.B	%00001100
	DC.B	%00000011
	DC.B	%00000011
	DC.B	%00001100
	DC.B	%00110000
	DC.B	%11000000
	DC.B	%11000000
	DC.B	%00110000
	DC.B	%00001100
	DC.B	%00000011
	DC.B	%00000011
	DC.B	%00001100
	DC.B	%00110000
	DC.B	%11000000	
Shifts
	DC.B	3,2,1,0,0,1,2,3,3,2,1,0,0,1,2,3
		       	 	
        ; Screen Data
        ORG ($FC00 + KERNH)
	
	; Block Data For Screen
BlockData
; Block Data - 8 Bytes:	
; Byte 0 - Map Data (00=Empty, 01=Wall, 10=Ladder, 11=Reserved)
; Bytes 1-2 - Ladder Data
; Byte 3 - Blanking Line
; Bytes 4-7 - Block Data
Block0L	
Block0R
	DC.B	%00000000
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00000000	
Block1L
	DC.B	%01010101
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%11111111
	DC.B	%01010101
	DC.B	%10101010
	DC.B	%11111111
Block1R
	DC.B	%01010101
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%11111111
	DC.B	%10101010		
	DC.B	%01010101
	DC.B	%11111111
Block2L
	DC.B	%01011001
	DC.B	%00001100	
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%11110011
	DC.B	%01011101
	DC.B	%10100010
	DC.B	%11110011
Block2R
	DC.B	%01100101
	DC.B	%00110000	
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%11001111
	DC.B	%10111010		
	DC.B	%01000101
	DC.B	%11001111
Block3L
	DC.B	%00001000
	DC.B	%00001100	
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%00001100
	DC.B	%00000000
	DC.B	%00000000
Block3R
	DC.B	%00100000
	DC.B	%00110000	
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%00110000
	DC.B	%00000000
	DC.B	%00000000			
Block4L
	DC.B	%00010000
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%00110000
	DC.B	%00010000
	DC.B	%00100000
	DC.B	%00110000
Block4R
	DC.B	%00000100
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%00001100
	DC.B	%00001000		
	DC.B	%00000100
	DC.B	%00001100
Block5L
	DC.B	%00000001
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%00000011
	DC.B	%00000001
	DC.B	%00000010
	DC.B	%00000011
Block5R
	DC.B	%01000000
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%11000000
	DC.B	%10000000		
	DC.B	%01000000
	DC.B	%11000000
Block6L
	DC.B	%00000101
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%00001111
	DC.B	%00000101
	DC.B	%00001010
	DC.B	%00001111
Block6R
	DC.B	%01010000
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%11110000
	DC.B	%10100000		
	DC.B	%01010000
	DC.B	%11110000
Block7L
	DC.B	%01000100
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%11001100
	DC.B	%01000100
	DC.B	%10001000
	DC.B	%11001100
Block7R
	DC.B	%00010001
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%00110011
	DC.B	%00100010		
	DC.B	%00010001
	DC.B	%00110011		
Block8L
	DC.B	%00010101
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%00111111
	DC.B	%00010101
	DC.B	%00101010
	DC.B	%00111111
Block8R
	DC.B	%01010100
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%11111100
	DC.B	%10101000		
	DC.B	%01010100
	DC.B	%11111100	
Block9L
	DC.B	%01010100
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%11111100
	DC.B	%01010100
	DC.B	%10101000
	DC.B	%11111100
Block9R
	DC.B	%00010101
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%00111111
	DC.B	%00101010		
	DC.B	%00010101
	DC.B	%00111111	
EndBlockData
        if (>BlockData != >EndBlockData)
          echo "WARNING: Block Data crosses a page boundary!"
        endif		
	       	 
        ; HMOVE Fine Tune Table
        ORG $FD00,0
FineTune
	; DC.B	%01110010	
        DC.B    %01100010       ; + 6 (Left)
        DC.B    %01010010       ; + 5
        DC.B    %01000010
        DC.B    %00110010
        DC.B    %00100010
        DC.B    %00010010
        DC.B    %00000010       ; Centre
        DC.B    %11110010
        DC.B    %11100010
        DC.B    %11010010
        DC.B    %11000010
        DC.B    %10110010
        DC.B    %10100010
        DC.B    %10010010       ; - 7
        DC.B    %10000010       ; - 8 (Right)
FineTuneEnd = FineTune - 241
			
        ; Sprite Data
        ORG ($FE00 + KERNH)
StartSprites

	; Player Sprites
HorizFrames
	DC.B	<Player0,<Player1,<Player0,<Player2
VertFrames
	DC.B	<Player3,<Player4
Player0
        DC.B	%00000000
        DC.B	%00011100
        DC.B	%00011000
        DC.B	%00111100
        DC.B	%01111110
        DC.B	%01101110
        DC.B	%01101100
        DC.B	%01111100
        DC.B	%00111010
        DC.B	%00011110
        DC.B	%00001100
Player1
        DC.B	%00000000
        DC.B	%00100100
        DC.B	%01011110
        DC.B	%00111100
        DC.B	%01111110
        DC.B	%01110110
        DC.B	%01101100
        DC.B	%01111100
        DC.B	%00111010
        DC.B	%00011110
        DC.B	%00001100
Player2
        DC.B	%00000000
        DC.B	%00100100
        DC.B	%01111010
        DC.B	%00111100
        DC.B	%00111110
        DC.B	%01011110
        DC.B	%01101100
        DC.B	%01111100
        DC.B	%00111010
        DC.B	%00011110
        DC.B	%00001100
Player3
        DC.B	%00000000
        DC.B	%01110000
        DC.B	%00101110
        DC.B	%00110100
        DC.B	%00111100
        DC.B	%01111110
        DC.B	%01011110
        DC.B	%00111010
        DC.B	%01010100
        DC.B	%01111100
        DC.B	%00111000
Player4
        DC.B	%00000000
        DC.B	%00001110
        DC.B	%01110100
        DC.B	%00101100
        DC.B	%01111100
        DC.B	%01111110
        DC.B	%01111010
        DC.B	%01011100
        DC.B	%00101010
        DC.B	%00111110
        DC.B	%00011100

	; Knight Sprites
KnightFrames
	DC.B	<Knight0,<Knight1
Knight0
        DC.B	%00000000
        DC.B	%10110110
        DC.B	%10010010
        DC.B	%10111110
        DC.B	%10101111
        DC.B	%10101111
        DC.B	%10000111
        DC.B	%10101111
        DC.B	%10111110
        DC.B	%11010110
        DC.B	%10111110
        DC.B	%00011100
Knight1
        DC.B	%00000000
        DC.B	%00011100
        DC.B	%10001100
        DC.B	%10111110
        DC.B	%10101111
        DC.B	%10101111
        DC.B	%10000111
        DC.B	%10101111
        DC.B	%10111110
        DC.B	%10010110
        DC.B	%11111110
        DC.B	%10011100
	
	; Bell Sprites (Padded to 8 bytes)
	; NOTE:	HMove Timings are Delayed By 1
BellCycle
	DC.B	<Bell1,<Bell2,<Bell3,<Bell4,<Bell4,<Bell3,<Bell2,<Bell1
Bell1
	DC.B	%00000000	
	DC.B	%00000000			
	DC.B	%00000000	
	DC.B	%11100010
        DC.B    %00101110
        DC.B    %00001010
        DC.B    %00011010
        DC.B    %00000110
Bell2
	DC.B	%00000000	
	DC.B	%00000000			
	DC.B	%00000000	
	DC.B	%11010010
        DC.B    %00101110
        DC.B    %00001010
        DC.B    %00011010
        DC.B    %00000110
Bell3
	DC.B	%00000000	
	DC.B	%00000000			
	DC.B	%00000000
	DC.B	%11000010
        DC.B    %00101110
        DC.B    %00001010
        DC.B    %00011010
        DC.B    %00000110		
Bell4
	DC.B	%00000000	
	DC.B	%00000000			
	DC.B	%00000000	
	DC.B	%10110010
        DC.B    %00101110
        DC.B    %00001010
        DC.B    %00011010
        DC.B    %00000110

	; Missile Sprites		
Missile1				
	DC.B	%00000000			
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00000100	
	DC.B	%00010110
	DC.B	%11110110
	DC.B	%00000000
	DC.B	%00000000	
Missile2
	DC.B	%00000000			
	DC.B	%00000000		
	DC.B	%00000000
	DC.B	%00000100	
	DC.B	%11110110
	DC.B	%00010110
	DC.B	%00000000
	DC.B	%00000000
Missile3
	DC.B	%00000000			
	DC.B	%00000000		
	DC.B	%00000000
	DC.B	%00000100	
	DC.B	%00010110
	DC.B	%00010110
	DC.B	%00000000
	DC.B	%00000000	
EndSprites
        if (>StartSprites != >EndSprites)
          echo "WARNING: Sprite Data crosses a page boundary!"
        endif

End
        DC.B    "Hunchy II - (C) Chris Walton 2005"    

        ORG $FFFC,0
        .word Start
        .word Start
